#include "includes.h"

void main(void)
{
	unsigned char name[16];
	unsigned char* buffer = NULL;
	int size = 0, real_size;
	unsigned int key = 0;
	int error;

	Bdisp_AllClr_VRAM();
	print_infos();

	GetKey(&key);

	if(key == KEY_CTRL_F1)
	{
		input_nb(&size, (unsigned char*)"ENTER FILE SIZE:");
		
		real_size = (size % 4 == 0) ? size : size + (4 - size%4);

		if(real_size > 10000)
		{
			print_msg((unsigned char*)"ERROR!", (unsigned char*)"File is too big");
			return;
		}

		input_txt(name, (unsigned char*)"ENTER FILE NAME:");

		buffer = (unsigned char*)malloc(real_size);

		if(buffer == NULL)
		{
			print_msg((unsigned char*)"ERROR!", (unsigned char*)"Not enough RAM");
			return;
		}

		error = get_file(buffer, size);

		if(error != 0)
			return;
		
		create_file(buffer, real_size, name);

		free(buffer);
	
	}
}

int get_file(unsigned char* buffer, int size)
{
	unsigned char config[] = {0, 9, 0, 0, 0, 0};
	short data;
	int error;
	int data_size;
	int i = 0;

	print_loading_box((unsigned char*)"LOADING...");

	while(Serial_IsOpen() != 1) Serial_Open(config);

	Serial_ClearReceiveBuffer();

	while(i < size)
	{
		print_loading_bar(i, size);

		data_size = Serial_GetRxBufferSize();
		
		if(data_size > 0)
		{
			error = Serial_ReadBytes(buffer + i, data_size, &data);
		
			if(error == 0)
			{
				i += data;
			}
		}

		Sleep(20);

		if(IsKeyDown(KEY_CTRL_AC))
		{
			print_msg((unsigned char*)"ERROR!", (unsigned char*)"Process aborted");
			return 1;
		}
	}

	while(Serial_IsOpen() != 3) Serial_Close(0);

	return 0;
}

void create_file(unsigned char *buffer, int size, unsigned char *name)
{
	int handle, error;
	unsigned char complete_name[20];

	memcpy(complete_name, "\\\\fls0\\", 7);
	memcpy(complete_name + 7, name, 13);

	if(memory_exists((char*)complete_name))
		memory_deletefile((char*)complete_name);

	error = memory_createfile((char*)complete_name, size);

	switch(error)
	{
		case IML_FILEERR_NOERROR : break;

		case IML_FILEERR_ILLEGALPARAM : print_msg((unsigned char*)"ERROR!", (unsigned char*)"Illegal parameters");
			return;
		case IML_FILEERR_ILLEGALPATH : print_msg((unsigned char*)"ERROR!", (unsigned char*)"Illegal path");
			return;
		case IML_FILEERR_ACCESSDENYED : print_msg((unsigned char*)"ERROR!", (unsigned char*)"Access denyed");
			return;
		case IML_FILEERR_DEVICEFULL: print_msg((unsigned char*)"ERROR!", (unsigned char*)"Memory full");
			return;
		default : print_msg((unsigned char*)"ERROR!", (unsigned char*)"Unknow error");
			return;
	}
	
	handle = memory_openfile((char*)complete_name, FILE_READWRITE);

	if(handle < 0)
	{
		print_msg((unsigned char*)"ERROR!", (unsigned char*)"Unknow error");
		return;
	}

	error = memory_writefile(handle, (char*)buffer, size);

	if(error < 0)
	{
		print_msg((unsigned char*)"ERROR!", (unsigned char*)"Unknow error");
		memory_closefile(handle);
		return;
	}

	memory_closefile(handle);

	print_msg((unsigned char*)"SUCCESS!", (unsigned char*)"All is done !");
}

void input_txt(unsigned char* txt, unsigned char* title) // need a 13-long text buffer
{
	int i = 0;
	unsigned int key = 0;
	
	for(i=0; i<13; i++) txt[i] = 0;

	KBD_PutKey(KEY_CTRL_SHIFT, 0);
	GetKey(&key);
	KBD_PutKey(KEY_CTRL_ALPHA, 0);
	GetKey(&key);
	Cursor_SetFlashStyle(2);
	Cursor_SetFlashMode(1);
	Cursor_SetPosition(5, 4);

	i = 0;

	while(key != KEY_CTRL_EXE)
	{
		PopUpWin(4);
		locate(11 - str_len((char*)title)/2, 2);
		Print(title);
		locate(5, 4);
		Print(txt);

		GetKey(&key);

		if(i < 12)
		{
			if((key >= 'A' && key <= 'Z') || (key >= '0' && key <= '9'))
			{
				txt[i] = key;
				i++;
			}

			if(key == ' ' || key == '.')
			{
				txt[i] = '.';
				i++;
			}
		}

		if(key == KEY_CTRL_DEL)
		{
			i--;
			txt[i] = 0;
		}

		if(i < 0) i = 0;
		if(i == 12) txt[i] = 0;
		if(i > 12) i = 12;
	}

	Cursor_SetFlashMode(0);
	Cursor_SetFlashStyle(0);
}

void input_nb(int* nb, unsigned char* title)
{
	unsigned char txt[13];
	int i = 0;
	unsigned int key = 0;
	
	for(i=0; i<13; i++) txt[i] = 0;

	GetKey(&key);
	Cursor_SetFlashStyle(0);
	Cursor_SetFlashMode(1);
	Cursor_SetPosition(5,4);

	i = 0;

	while(key != KEY_CTRL_EXE)
	{
		PopUpWin(4);
		locate(11 - str_len((char*)title)/2, 2);
		Print(title);
		locate(5, 4);
		Print(txt);

		GetKey(&key);

		if(i < 12)
		{
			if(key >= KEY_CHAR_0 && key <= KEY_CHAR_9)
			{
				txt[i] = key;
				i++;
			}
		}

		if(key == KEY_CTRL_DEL)
		{
			i--;
			txt[i] = 0;
		}

		if(i < 0) i = 0;
		if(i == 12) txt[i] = 0;
		if(i > 12) i = 12;
	}

	Cursor_SetFlashMode(0);

	*nb = atoi(txt);
}

void print_msg(unsigned char* title, unsigned char* error)
{
	unsigned int key = 0;
	
	PopUpWin(4);
	locate(11 - str_len((char*)title)/2, 2);
	Print(title);
	locate(11 - str_len((char*)error)/2, 3);
	Print(error);
	locate(5, 5);
	Print((unsigned char*)"Press [EXIT]");

	while(key != KEY_CTRL_EXIT) GetKey(&key);
}

void print_infos(void)
{
	locate(1, 1);
	Print((unsigned char*)"DLFiles by Dark Storm");
	locate(1, 3);
	Print((unsigned char*)"Speed :115200 bauds");
	locate(1, 4);
	Print((unsigned char*)"Size max : 10ko");
	locate(1, 8);
	Print((unsigned char*)"F1: Start download");
}

void print_loading_box(unsigned char* title)
{
	PopUpWin(3);
	locate(11 - str_len((char*)title)/2, 3);
	Print(title);

	_draw_rect(24, 27, 104, 37);

	Bdisp_PutDisp_DD();
}

void print_loading_bar(int pos, int max)
{
	int i;
	for(i = 0; i < pos*80/max; i++)
		Bdisp_DrawLineVRAM(i + 24, 27, i + 24, 37);
	Bdisp_PutDisp_DD();
}

void _draw_rect(int x1, int y1, int x2, int y2)
{
	Bdisp_DrawLineVRAM(x1, y1, x2, y1);
	Bdisp_DrawLineVRAM(x1, y1, x1, y2);
	Bdisp_DrawLineVRAM(x2, y2, x2, y1);
	Bdisp_DrawLineVRAM(x2, y2, x1, y2);
}